.BG
.FN glim
.TL
Fit a Generalized Linear Model
.SH DESCRIPTION
Fit a generalized linear model of the form y = f(Xb) + error via
maximum likelihood.
.CS
glim(x, y, n, error="gaussian", link="identity", wt=1, resid="none",
init, intercept=T, scale, offset=0, sequence, eps=0.0001, iter.max=10)
.AG x
matrix of predictor variables, with variables in columns and observations
across rows.
.AG y
vector containing the dependent variable.
The length of `y' should equal the number of rows of `x'.
.AG n
when `error' is "binomial", `y' should be a count
and `n' the vector of denominators for the proportion.
Should have the same length as `y'.
When `error' is not "binomial" then `n' is ignored.
.AG wt
optional vector of case weights.
Should have the same length as `y'.
.AG error
a character string specifying the error structure for the model.
Possible values are "gaussian", "binomial", "poisson", "gamma"
and "inverse gaussian".
Only enough of the string to determine a unique match is required.
.AG link
a character string specifying the link function, which is the inverse of
f above.
Possible values are "identity", "log", "logit", "probit", "sqrt", "inverse",
and "loglog".
Definitions of g(y) are y, log(y), log(y/(1-y)), pnorm(y), sqrt(y), 1/y,
and log(-log(1-y)) respectively.
Only enough of the string to determine a unique match is required.
.AG resid
a character string specifying the type of residual desired.
Possible values are "none", "pearson", and "deviance".
Only enough of the string to determine a unique match is required.
.AG init
optional vector of initial values for the model coefficients.
Must be of length `1 + ncol(x)' if `intercept' is TRUE.
.AG intercept
if TRUE a constant (intercept) term is included in the model.
.AG scale
the scale factor for the model.
May be either a numeric constant or a string specifying the estimation
method, either "deviance" or "pearson".
The default value is 1 for "binomial" and "poisson" error structures,
and "pearson" for the others.
.AG offset
in certain cases a predictor variable with a fixed coefficient needs to be
included in the model.  This is modeled as y = f(Xb + offset), where b
is the vector of estimated coefficients, and the variable or variables
with fixed coefficient have been left out of the X matrix and summed
into a vector 'offset'.
.AG sequence
An integer vector giving the sequence of models to be fit.
It must be in increasing order.
A value of `1:3', for instance, would cause models to be fit using the
first column of x, then the first 2 columns, then the first 3 columns.
The default value is `c(0, ncol(x))'.
If `intercept = TRUE', this means that the first model is a fit of only the
mean or mean+offset terms.
.AG eps
iteration will stop when the relative change in log likelihood is less
than `eps' or after `iter.max' iterations.
.AG iter.max
maximum number of iterations.
.AG pred
if TRUE a vector of predicted values will be returned.
.RT
a list with components:
.AG coef
the vector of estimated coefficients for the final model that was fit.
.AG var
estimated variance matrix of `coef'
(based on the expected Fisher information matrix).
It has been adjusted using the value of `scale'.
.AG deviance
a vector of deviance values for the sequence of models.
The deviance is not divided by the scale estimate
(otherwise a gaussian model would always have deviance of
`nrow(x)' - #variables).
.AG df
a vector of degrees of freedom corresponding to the values in `deviance'.
This takes account of any observations with a zero weight.
.AG scale
the scale value of the final model,
either as estimated or the fixed value that was input.
.AG intercept
a copy of the input argument.
.AG pred
vector of predicted values.
.AG resid
vector of standardized residuals for the final model.
The residual has not been standardized (multiplied) by the `wt' vector.
This is the correct behavior if, say, the weights were integers representing
frequency for each observation.
For deviance residuals, the overall deviance will be
`scale * sum(wt * resid^2)'.
.SH REFERENCE
McCullagh, P. and Nelder, J. A. (1983).
.i
Generalized Linear Models.
Chapman and Hall, London.
.SA
`glim.prt'
.EX
.KW regression glim
.WR
