estimate.contrasts.I := function (contrast.matrix, anova, alpha = 0.05) 
{
	if (alpha < 0 || alpha > 1) 
		alpha <- 0.05
	k <- length(anova$sizes)
	contrast.matrix <- if (!is.matrix(contrast.matrix)) 
		matrix(contrast.matrix, ncol = length(contrast.matrix))
	else contrast.matrix
	if (ncol(contrast.matrix) != k) 
		stop("Wrong number of columns in contrast matrix")
	means <- anova$means
	contrasts <- contrast.matrix %*% means
	qqt <- qt(1 - alpha/(2 * length(contrasts)), anova$df)
	stderr <- sqrt(contrast.matrix^2 %*% (1/anova$sizes)) * anova$rms
	tstat <- round(contrasts/stderr, 4)
	pval <- round(2 * (1 - pt(abs(tstat), anova$df)), 4)
	ll <- round(contrasts - stderr * qqt, 4)
	ul <- round(contrasts + stderr * qqt, 4)
	outmat <- cbind(contrasts, stderr, tstat, pval, ll, ul)
	contrast.names <- if (is.null(dimnames(contrast.matrix))) 
		paste("Contrast", 1:length(contrasts))
	else dimnames(contrast.matrix)[[1]]
	dimnames(outmat) <- list(contrast.names, c("Estimate", "Std error", "T-value", 
		"p-value", "lowerlimit", "upperlimit"))
	outmat
}
